// Generated automatically.  DO NOT HAND-EDIT.

package d200

import "github.com/gdamore/tcell/terminfo"

func init() {

	// Data General DASHER D200
	terminfo.AddTerminfo(&terminfo.Terminfo{
		Name:        "d200",
		Aliases:     []string{"d200-dg"},
		Columns:     80,
		Lines:       24,
		Bell:        "\a",
		Clear:       "\f",
		AttrOff:     "\x0f\x15\x1d\x1eE",
		Underline:   "\x14",
		Bold:        "\x1eD\x14",
		Dim:         "\x1c",
		Blink:       "\x0e",
		Reverse:     "\x1eD",
		PadChar:     "\x00",
		SetCursor:   "\x10%p2%c%p1%c",
		CursorBack1: "\x19",
		CursorUp1:   "\x17",
		KeyUp:       "\x17",
		KeyDown:     "\x1a",
		KeyRight:    "\x18",
		KeyLeft:     "\x19",
		KeyHome:     "\b",
		KeyF1:       "\x1eq",
		KeyF2:       "\x1er",
		KeyF3:       "\x1es",
		KeyF4:       "\x1et",
		KeyF5:       "\x1eu",
		KeyF6:       "\x1ev",
		KeyF7:       "\x1ew",
		KeyF8:       "\x1ex",
		KeyF9:       "\x1ey",
		KeyF10:      "\x1ez",
		KeyF11:      "\x1e{",
		KeyF12:      "\x1e|",
		KeyF13:      "\x1e}",
		KeyF14:      "\x1e~",
		KeyF15:      "\x1ep",
		KeyF16:      "\x1ea",
		KeyF17:      "\x1eb",
		KeyF18:      "\x1ec",
		KeyF19:      "\x1ed",
		KeyF20:      "\x1ee",
		KeyF21:      "\x1ef",
		KeyF22:      "\x1eg",
		KeyF23:      "\x1eh",
		KeyF24:      "\x1ei",
		KeyF25:      "\x1ej",
		KeyF26:      "\x1ek",
		KeyF27:      "\x1el",
		KeyF28:      "\x1em",
		KeyF29:      "\x1en",
		KeyF30:      "\x1e`",
		KeyF31:      "\x1e1",
		KeyF32:      "\x1e2",
		KeyF33:      "\x1e3",
		KeyF34:      "\x1e4",
		KeyF35:      "\x1e5",
		KeyF36:      "\x1e6",
		KeyF37:      "\x1e7",
		KeyF38:      "\x1e8",
		KeyF39:      "\x1e9",
		KeyF40:      "\x1e:",
		KeyF41:      "\x1e;",
		KeyF42:      "\x1e<",
		KeyF43:      "\x1e=",
		KeyF44:      "\x1e>",
		KeyF45:      "\x1e0",
		KeyF46:      "\x1e!",
		KeyF47:      "\x1e\"",
		KeyF48:      "\x1e#",
		KeyF49:      "\x1e$",
		KeyF50:      "\x1e%%",
		KeyF51:      "\x1e&",
		KeyF52:      "\x1e'",
		KeyF53:      "\x1e(",
		KeyF54:      "\x1e)",
		KeyF55:      "\x1e*",
		KeyF56:      "\x1e+",
		KeyF57:      "\x1e,",
		KeyF58:      "\x1e-",
		KeyF59:      "\x1e.",
		KeyF60:      "\x1e ",
		KeyClear:    "\f",
		KeyShfLeft:  "\x1e\x19",
		KeyShfRight: "\x1e\x18",
		KeyShfHome:  "\x1e\b",
	})
}
